@@ -10,40 +10,45 @@ from django.contrib import admin
 from django.db.models import Model
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404
+from django.utils import six
+from django.utils.encoding import smart_text
 from django.utils.translation import ugettext_lazy as _
 
 from form_designer import models
 
 
-class UnicodeWriter:
-    """
-    A CSV writer which will write rows to CSV file "f",
-    which is encoded in the given encoding.
-    """
+if six.PY3:
+    UnicodeWriter = csv.writer
+else:
+    class UnicodeWriter:
+        """
+        A CSV writer which will write rows to CSV file "f",
+        which is encoded in the given encoding.
+        """
 
-    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
-        # Redirect output to a queue
-        self.queue = BytesIO()
-        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
-        self.stream = f
-        self.encoder = codecs.getincrementalencoder(encoding)('replace')
+        def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
+            # Redirect output to a queue
+            self.queue = BytesIO()
+            self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
+            self.stream = f
+            self.encoder = codecs.getincrementalencoder(encoding)('replace')
 
-    def writerow(self, row):
-        row = [unicode(s) for s in row]
-        self.writer.writerow([s.encode("utf-8") for s in row])
-        # Fetch UTF-8 output from the queue ...
-        data = self.queue.getvalue()
-        data = data.decode("utf-8")
-        # ... and reencode it into the target encoding
-        data = self.encoder.encode(data)
-        # write to the target stream
-        self.stream.write(data)
-        # empty queue
-        self.queue.truncate(0)
+        def writerow(self, row):
+            row = [smart_text(s) for s in row]
+            self.writer.writerow([s.encode("utf-8") for s in row])
+            # Fetch UTF-8 output from the queue ...
+            data = self.queue.getvalue()
+            data = data.decode("utf-8")
+            # ... and reencode it into the target encoding
+            data = self.encoder.encode(data)
+            # write to the target stream
+            self.stream.write(data)
+            # empty queue
+            self.queue.truncate(0)
 
-    def writerows(self, rows):
-        for row in rows:
-            self.writerow(row)
+        def writerows(self, rows):
+            for row in rows:
+                self.writerow(row)
 
 
 def jsonize(v):
@@ -60,7 +65,7 @@ class FormAdminForm(forms.ModelForm):
     class Meta:
         widgets = {
             'config_json': forms.Textarea(attrs={'rows': 3}),
-            }
+        }
 
     def __init__(self, *args, **kwargs):
         super(FormAdminForm, self).__init__(*args, **kwargs)
@@ -74,7 +79,7 @@ class FormAdminForm(forms.ModelForm):
             label=_('Options'),
             help_text=_('Save and continue editing to configure options.'),
             widget=forms.CheckboxSelectMultiple,
-            )
+        )
 
         config_fieldsets = []
 
@@ -95,7 +100,7 @@ class FormAdminForm(forms.ModelForm):
             fieldset = [
                 _('Form configuration: %s') % cfg.get('title', s),
                 {'fields': []},
-                ]
+            ]
 
             for k, f in cfg.get('form_fields', []):
                 self.fields['%s_%s' % (s, k)] = f
@@ -156,9 +161,12 @@ class FormAdmin(admin.ModelAdmin):
 
         fieldsets[0][1]['fields'].remove('config_json')
 
-        fieldsets.append((_('Configuration'), {
-            'fields': ('config_json', 'config_options'),
-            }))
+        fieldsets.append((
+            _('Configuration'),
+            {
+                'fields': ('config_json', 'config_options'),
+            }
+        ))
 
         fieldsets.extend(request._formdesigner_config_fieldsets)
 
@@ -171,7 +179,7 @@ class FormAdmin(admin.ModelAdmin):
         for submission in form.submissions.all():
             data = submission.sorted_data(include=('date', 'time', 'path'))
             if not rows:
-                rows.append(data.keys())
+                rows.append(list(data.keys()))
             rows.append([data.get(field_name) for field_name in rows[0]])
             # (fairly gracefully handles changes in form fields between
             #  submissions)
@@ -187,9 +195,11 @@ class FormAdmin(admin.ModelAdmin):
     def get_urls(self):
         return patterns(
             '',
-            url(r'(?P<form_id>\d+)/export_submissions/',
-                self.admin_site.admin_view(self.export_submissions))
-            ) + super(FormAdmin, self).get_urls()
+            url(
+                r'(?P<form_id>\d+)/export_submissions/',
+                self.admin_site.admin_view(self.export_submissions),
+            )
+        ) + super(FormAdmin, self).get_urls()
 
 
 class FormSubmissionAdmin(admin.ModelAdmin):
