@@ -1,14 +1,18 @@
 from django import forms
-from django.conf import settings
+from django.conf import settings as django_settings
 from django.core.mail import send_mail
 from django.db import models
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.template import RequestContext
 from django.template.loader import render_to_string
 from django.utils.datastructures import SortedDict
-from django.utils.functional import curry
+from django.utils.encoding import python_2_unicode_compatible, smart_text
 from django.utils.translation import ugettext_lazy as _
 
+from form_designer import utils
+from form_designer import settings
+
+
 try:
     from django.utils.text import slugify
 except ImportError:  # Django 1.4
@@ -36,12 +40,16 @@ def send_as_mail(model_instance, form_instance, request, config, **kwargs):
         data=repr(form_instance.cleaned_data),
         path=request.path)
 
-    send_mail(model_instance.title, submission.formatted_data(),
-              settings.DEFAULT_FROM_EMAIL,
-              [config['email']], fail_silently=True)
+    send_mail(
+        model_instance.title,
+        submission.formatted_data(),
+        django_settings.DEFAULT_FROM_EMAIL,
+        [config['email']],
+        fail_silently=True)
     return _('Thank you, your input has been received.')
 
 
+@python_2_unicode_compatible
 class Form(models.Model):
     CONFIG_OPTIONS = [
         ('save_fs', {
@@ -66,14 +74,14 @@ class Form(models.Model):
         verbose_name = _('form')
         verbose_name_plural = _('forms')
 
-    def __unicode__(self):
+    def __str__(self):
         return self.title
 
     def form(self):
         fields = SortedDict((
             ('required_css_class', 'required'),
             ('error_css_class', 'error'),
-            ))
+        ))
 
         for field in self.fields.all():
             field.add_formfield(fields, self)
@@ -115,38 +123,13 @@ class Form(models.Model):
         return ret
 
 
+FIELD_TYPES = utils.get_object(settings.FORM_DESIGNER_FIELD_TYPES)
+
+
+@python_2_unicode_compatible
 class FormField(models.Model):
-    FIELD_TYPES = [
-        ('text', _('text'), forms.CharField),
-        ('email', _('e-mail address'), forms.EmailField),
-        ('longtext', _('long text'),
-         curry(forms.CharField, widget=forms.Textarea)),
-        ('checkbox', _('checkbox'), curry(forms.BooleanField, required=False)),
-        ('select', _('select'), curry(forms.ChoiceField, required=False)),
-        ('radio', _('radio'),
-         curry(forms.ChoiceField, widget=forms.RadioSelect)),
-        ('multiple-select', _('multiple select'), curry(
-            forms.MultipleChoiceField,
-            widget=forms.CheckboxSelectMultiple)),
-        ('hidden', _('hidden'), curry(
-            forms.CharField,
-            widget=forms.HiddenInput)),
-    ]
-
-    # Add recaptcha field if available
-    if 'captcha' in settings.INSTALLED_APPS:
-        try:
-            from captcha.fields import ReCaptchaField
-        except ImportError:
-            pass
-        else:
-            FIELD_TYPES.append(
-                ('recaptcha', _('recaptcha'),
-                 curry(ReCaptchaField, attrs={'theme': 'clean'})),
-            )
-
-    form = models.ForeignKey(Form, related_name='fields',
-        verbose_name=_('form'))
+    form = models.ForeignKey(
+        Form, related_name='fields', verbose_name=_('form'))
     ordering = models.IntegerField(_('ordering'), default=0)
 
     title = models.CharField(_('title'), max_length=100)
@@ -170,7 +153,7 @@ class FormField(models.Model):
         verbose_name = _('form field')
         verbose_name_plural = _('form fields')
 
-    def __unicode__(self):
+    def __str__(self):
         return self.title
 
     def clean(self):
@@ -186,15 +169,18 @@ class FormField(models.Model):
         return tuple(choices)
 
     def get_type(self, **kwargs):
-        types = dict((r[0], r[2]) for r in self.FIELD_TYPES)
+        types = dict((r[0], r[2]) for r in FIELD_TYPES)
         return types[self.type](**kwargs)
 
     def add_formfield(self, fields, form):
         fields[slugify(self.name)] = self.formfield()
 
     def formfield(self):
-        kwargs = dict(label=self.title, required=self.is_required,
-            initial=self.default_value)
+        kwargs = dict(
+            label=self.title,
+            required=self.is_required,
+            initial=self.default_value,
+        )
         if self.choices:
             kwargs['choices'] = self.get_choices()
         if self.help_text:
@@ -220,7 +206,7 @@ class FormSubmission(models.Model):
         `include` can be a tuple containing any or all of 'date', 'time',
         'datetime', or 'path' to include additional meta data.
         """
-        data_dict = eval(self.data)
+        data_dict = eval(self.data)  # XXX fix this! eval() !?
         data = SortedDict()
         field_names = []
         for field in self.form.fields.all():
@@ -278,9 +264,12 @@ class FormContent(models.Model):
         """ Process form and return response (hook method). """
         process_result = self.form.process(form_instance, request)
         context = RequestContext(
-            request, {
+            request,
+            {
                 'content': self,
-                'message': self.success_message or process_result or u''})
+                'message': self.success_message or process_result or u'',
+            }
+        )
         return render_to_string(self.template, context)
 
     def render(self, request, **kwargs):
@@ -289,7 +278,7 @@ class FormContent(models.Model):
         formcontent = request.POST.get('_formcontent')
 
         if request.method == 'POST' and (
-                not formcontent or formcontent == unicode(self.id)):
+                not formcontent or formcontent == smart_text(self.id)):
             form_instance = form_class(request.POST, prefix=prefix)
 
             if form_instance.is_valid():
